@@ -39,7 +39,7 @@ import time
 from itertools import product
 
 import numpy as np
-from dlg.common import Categories, DropType
+from dlg.common import Categories, CategoryType, DropType
 from dlg.common import STORAGE_TYPES, APP_DROP_TYPES
 from dlg.common import dropdict
 from dlg.dropmake.dm_utils import (
@@ -147,10 +147,14 @@ class LGNode:
             return self.group.id
 
     def add_output(self, lg_node):
-        self._outs.append(lg_node)
+        if lg_node not in self._outs:
+            self._outs.append(lg_node)
 
     def add_input(self, lg_node):
-        self._inputs.append(lg_node)
+        # only add if not already there
+        # this may happen in nested constructs
+        if lg_node not in self._inputs:
+            self._inputs.append(lg_node)
 
     def add_child(self, lg_node):
         """
@@ -163,7 +167,7 @@ class LGNode:
             and not (lg_node.is_groupby())
         ):
             raise GInvalidNode(
-                "Only Scatters or Loops can be nested, but {0} is neither".format(
+                "Only Scatters, Loops and GroupBys can be nested, but {0} is neither".format(
                     lg_node.id
                 )
             )
@@ -398,7 +402,9 @@ class LGNode:
             return self._grpw
         else:
             raise GraphException(
-                "Non-GroupBy LGN {0} does not have groupby_width".format(self.id)
+                "Non-GroupBy LGN {0} does not have groupby_width".format(
+                    self.id
+                )
             )
 
     @property
@@ -424,11 +430,15 @@ class LGNode:
             # group by followed by another group by
             if grpks is None or len(grpks) < 1:
                 raise GInvalidNode(
-                    "Must specify group_key for Group By '{0}'".format(self.text)
+                    "Must specify group_key for Group By '{0}'".format(
+                        self.text
+                    )
                 )
             # find the "root" groupby and get all of its scatters
             inputgrp = self
-            while (inputgrp is not None) and inputgrp.inputs[0].group.is_groupby():
+            while (inputgrp is not None) and inputgrp.inputs[
+                0
+            ].group.is_groupby():
                 inputgrp = inputgrp.inputs[0].group
             # inputgrp now is the "root" groupby that follows Scatter immiately
             # move it to Scatter
@@ -529,44 +539,51 @@ class LGNode:
     def _update_key_value_attributes(self, kwargs):
         # get the arguments from new fields dictionary in a backwards compatible way
         if "fields" in self.jd:
+            self.jd.update({"nodeAttributes": {}})
+            kwargs.update({"nodeAttributes": {}})
             for je in self.jd["fields"]:
                 # The field to be used is not the text, but the name field
                 self.jd[je["name"]] = je["value"]
                 kwargs[je["name"]] = je["value"]
-        kwargs["applicationArgs"] = {} # make sure the dict always exists downstream
-        if "applicationArgs" in self.jd: # and fill it if provided
+                self.jd["nodeAttributes"].update({je["name"]: je})
+                kwargs["nodeAttributes"].update({je["name"]: je})
+        kwargs[
+            "applicationArgs"
+        ] = {}  # make sure the dict always exists downstream
+        if "applicationArgs" in self.jd:  # and fill it if provided
             for je in self.jd["applicationArgs"]:
-                j = {je["name"]:{k:je[k] for k in je if k not in ['name']}}
+                j = {je["name"]: {k: je[k] for k in je if k not in ["name"]}}
                 self.jd.update(j)
                 kwargs["applicationArgs"].update(j)
-        for i in range(10):
-            k = "Arg%02d" % (i + 1)
-            if k not in self.jd:
-                continue
-            v = self.jd[k]
-            if v is not None and len(str(v)) > 0:
-                for kv in v.split(","):  # comma separated k-v pairs
-                    k_v = kv.replace(" ", "").split("=")
-                    if len(k_v) > 1:
-                        # Do substitutions for MKN
-                        if "mkn" in self.jd:
-                            kwargs[k_v[0]] = self._mkn_substitution(
-                                self.jd["mkn"], k_v[1]
-                            )
-                        else:
-                            kwargs[k_v[0]] = k_v[1]
+        if "nodeAttributes" not in kwargs:
+            kwargs.update({"nodeAttributes": {}})
+        for k, na in kwargs["nodeAttributes"].items():
+            if (
+                "parameterType" in na
+                and na["parameterType"] == "ApplicationArgument"
+            ):
+                kwargs["applicationArgs"].update({k: na})
+        # NOTE: drop Argxx keywords
 
-    def _getIdText(self, port="inputPorts", index=0):
+    def _getIdText(self, port="inputPorts", index=0, portId=None):
         """
         Return IdText of port if it exists
         """
         idText = None
-        if (
-            port in self.jd
-            and len(self.jd[port]) > index
-            and "IdText" in self.jd[port][index]
-        ):
-            idText = self.jd[port][index]["IdText"]
+        if not portId:
+            if (
+                port in self.jd
+                and len(self.jd[port]) > index
+                and "IdText" in self.jd[port][index]
+            ):
+                idText = self.jd[port][index]["IdText"]
+        else:
+            if port in self.jd:
+                idText = [
+                    p["IdText"]
+                    for p in self.jd["inputPorts"]
+                    if p["Id"] == portId
+                ][0]
         return idText
 
     def _create_test_drop_spec(self, oid, rank, kwargs) -> dropdict:
@@ -578,6 +595,7 @@ class LGNode:
         drop_spec = None
         drop_type = self.jd["category"]
         if drop_type in STORAGE_TYPES:
+            logger.debug("Storage node spec: %s", json.dumps(kwargs))
             if "data_volume" in self.jd:
                 kwargs["dw"] = int(self.jd["data_volume"])  # dw -- data weight
             else:
@@ -589,7 +607,8 @@ class LGNode:
                 drop_spec = dropdict(
                     {
                         "oid": oid,
-                        "type": DropType.PLAIN,
+                        "type": DropType.DATA,
+                        "categoryType": CategoryType.DATA,
                         "storage": drop_type,
                         "rank": rank,
                     }
@@ -598,8 +617,10 @@ class LGNode:
                     {
                         "oid": "{0}-s".format(oid),
                         "type": DropType.APP,
+                        "categoryType": CategoryType.APPLICATION,
                         "app": "dlg.apps.simple.SleepApp",
                         "nm": "lstnr",
+                        "text": "lstnr",
                         "tw": 5,
                         "sleepTime": 5,
                         "rank": rank,
@@ -613,7 +634,8 @@ class LGNode:
                 drop_spec = dropdict(
                     {
                         "oid": oid,
-                        "type": DropType.PLAIN,
+                        "type": DropType.DATA,
+                        "categoryType": CategoryType.DATA,
                         "storage": drop_type,
                         "rank": rank,
                     }
@@ -660,7 +682,13 @@ class LGNode:
 
             kwargs["tw"] = execTime
             drop_spec = dropdict(
-                {"oid": oid, "type": DropType.APP, "app": app_class, "rank": rank}
+                {
+                    "oid": oid,
+                    "type": DropType.APP,
+                    "categoryType": CategoryType.APPLICATION,
+                    "app": app_class,
+                    "rank": rank,
+                }
             )
 
             kwargs["num_cpus"] = int(self.jd.get("num_cpus", 1))
@@ -671,11 +699,14 @@ class LGNode:
 
         elif drop_type in [Categories.DYNLIB_APP, Categories.DYNLIB_PROC_APP]:
             if "libpath" not in self.jd or len(self.jd["libpath"]) == 0:
-                raise GraphException("Missing 'libpath' in Drop {0}".format(self.text))
+                raise GraphException(
+                    "Missing 'libpath' in Drop {0}".format(self.text)
+                )
             drop_spec = dropdict(
                 {
                     "oid": oid,
                     "type": DropType.APP,
+                    "categoryType": CategoryType.APPLICATION,
                     "app": "dlg.apps.dynlib.{}".format(drop_type),
                     "rank": rank,
                 }
@@ -695,28 +726,27 @@ class LGNode:
             else:
                 app_str = "dlg.apps.bash_shell_app.BashShellApp"
             drop_spec = dropdict(
-                {"oid": oid, "type": DropType.APP, "app": app_str, "rank": rank}
+                {
+                    "oid": oid,
+                    "type": DropType.APP,
+                    "categoryType": CategoryType.APPLICATION,
+                    "app": app_str,
+                    "rank": rank,
+                }
             )
+            self._update_key_value_attributes(kwargs)
             if "execution_time" in self.jd:
-                kwargs["tw"] = int(self.jd["execution_time"])
+                try:
+                    kwargs["tw"] = int(self.jd["execution_time"])
+                except TypeError:
+                    kwargs["tw"] = int(self.jd["execution_time"]["value"])
             else:
                 # kwargs['tw'] = random.randint(3, 8)
                 raise GraphException(
                     "Missing execution_time for Construct '%s'" % self.text
                 )
-            # add more arguments
+            # add more arguments (support for Arg0x dropped!)
             cmds = []
-            for i in range(10):
-                k = "Arg%02d" % (i + 1,)
-                if k not in self.jd:
-                    k = "arg%02d" % (i + 1,)
-                    if k not in self.jd:
-                        continue
-                v = self.jd[k]
-                if v is not None and len(str(v)) > 0:
-                    cmds.append(str(v))
-            # add more arguments - this is the new method of adding arguments in EAGLE
-            # the method above (Arg**) is retained for compatibility, but eventually should be removed
             for k in [
                 "command",
                 "input_redirection",
@@ -732,7 +762,10 @@ class LGNode:
                 kwargs["command"] = BashCommand(
                     cmds
                 )  # TODO: Check if this actually solves a problem.
-            kwargs["num_cpus"] = int(self.jd.get("num_cpus", 1))
+            try:
+                kwargs["num_cpus"] = int(self.jd.get("num_cpus", 1))
+            except TypeError:
+                kwargs["num_cpus"] = int(self.jd["num_cpus"]["value"])
             drop_spec.update(kwargs)
 
         elif drop_type == Categories.DOCKER:
@@ -740,12 +773,20 @@ class LGNode:
             app_class = "dlg.apps.dockerapp.DockerApp"
             typ = DropType.APP
             drop_spec = dropdict(
-                {"oid": oid, "type": typ, "app": app_class, "rank": rank}
+                {
+                    "oid": oid,
+                    "type": typ,
+                    "categoryType": CategoryType.APPLICATION,
+                    "app": app_class,
+                    "rank": rank,
+                }
             )
 
             image = str(self.jd.get("image"))
             if image == "":
-                raise GraphException("Missing image for Construct '%s'" % self.text)
+                raise GraphException(
+                    "Missing image for Construct '%s'" % self.text
+                )
 
             command = str(self.jd.get("command"))
             # There ARE containers which don't need/want a command
@@ -762,9 +803,12 @@ class LGNode:
             kwargs["removeContainer"] = self.str_to_bool(
                 str(self.jd.get("removeContainer", "1"))
             )
-            kwargs["additionalBindings"] = str(self.jd.get("additionalBindings", ""))
+            kwargs["additionalBindings"] = str(
+                self.jd.get("additionalBindings", "")
+            )
             kwargs["portMappings"] = str(self.jd.get("portMappings", ""))
             kwargs["shmSize"] = str(self.jd.get("shmSize", ""))
+            self._update_key_value_attributes(kwargs)
             drop_spec.update(kwargs)
 
         elif drop_type == Categories.GROUP_BY:
@@ -772,6 +816,7 @@ class LGNode:
                 {
                     "oid": oid,
                     "type": DropType.APP,
+                    "categoryType": CategoryType.APPLICATION,
                     "app": "dlg.apps.simple.SleepApp",
                     "rank": rank,
                 }
@@ -786,15 +831,19 @@ class LGNode:
             dropSpec_grp = dropdict(
                 {
                     "oid": "{0}-grp-data".format(oid),
-                    "type": DropType.PLAIN,
+                    "type": DropType.DATA,
+                    "categoryType": CategoryType.DATA,
                     "storage": Categories.MEMORY,
                     "nm": "grpdata",
+                    "text": "grpdata",
                     "dw": dw,
                     "rank": rank,
                 }
             )
             kwargs["grp-data_drop"] = dropSpec_grp
-            kwargs["tw"] = 1  # barrier literarlly takes no time for its own computation
+            kwargs[
+                "tw"
+            ] = 1  # barrier literarlly takes no time for its own computation
             kwargs["sleepTime"] = 1
             drop_spec.addOutput(dropSpec_grp, IdText="grpdata")
             dropSpec_grp.addProducer(drop_spec, IdText="grpdata")
@@ -803,6 +852,7 @@ class LGNode:
                 {
                     "oid": oid,
                     "type": DropType.APP,
+                    "categoryType": CategoryType.APPLICATION,
                     "app": "dlg.apps.simple.SleepApp",
                     "rank": rank,
                 }
@@ -810,15 +860,21 @@ class LGNode:
             gi = self.inputs[0]
             if gi.is_groupby():
                 gii = gi.inputs[0]
-                dw = int(gii.jd["data_volume"]) * gi.groupby_width * self.gather_width
+                dw = (
+                    int(gii.jd["data_volume"])
+                    * gi.groupby_width
+                    * self.gather_width
+                )
             else:  # data
                 dw = int(gi.jd["data_volume"]) * self.gather_width
             dropSpec_gather = dropdict(
                 {
                     "oid": "{0}-gather-data".format(oid),
-                    "type": DropType.PLAIN,
+                    "type": DropType.DATA,
+                    "categoryType": CategoryType.DATA,
                     "storage": Categories.MEMORY,
                     "nm": "gthrdt",
+                    "text": "gthrdt",
                     "dw": dw,
                     "rank": rank,
                 }
@@ -832,30 +888,18 @@ class LGNode:
             raise GraphException(
                 f"DROP type: {drop_type} should not appear in physical graph"
             )
-            # drop_spec = dropdict(
-            #     {
-            #         "oid": oid,
-            #         "type": DropType.SERVICE_APP,
-            #         "app": "dlg.apps.simple.SleepApp",
-            #         "rank": rank
-            #     }
-            # )
-            # kwargs["tw"] = 1
-
-        # elif drop_type == Categories.BRANCH:
-        # Branches are now dealt with like any other application and essentially ignored by the translator.
-
         elif drop_type in [Categories.START, Categories.END]:
             # this is at least suspicious in terms of implementation....
             drop_spec = dropdict(
                 {
                     "oid": oid,
-                    "type": DropType.PLAIN,
+                    "type": DropType.DATA,
+                    "categoryType": CategoryType.DATA,
                     "storage": Categories.NULL,
                     "rank": rank,
                 }
             )
-        elif drop_type == Categories.LOOP:
+        elif drop_type in Categories.LOOP:
             pass
         else:
             raise GraphException("Unknown DROP type: '{0}'".format(drop_type))
@@ -873,7 +917,11 @@ class LGNode:
         kwargs["iid"] = iid
         kwargs["lg_key"] = self.id
         kwargs["dt"] = self.jd["category"]
+        kwargs["category"] = self.jd["category"]
+        if "categoryType" in kwargs:
+            kwargs["categoryType"] = self.jd["categoryType"]
         kwargs["nm"] = self.text
+        kwargs["text"] = self.text
         # Behaviour is that child-nodes inherit reproducibility data from their parents.
         if self._reprodata is not None:
             kwargs["reprodata"] = self._reprodata.copy()
@@ -925,7 +973,9 @@ class LG:
         lg = load_lg(f)
         if ssid is None:
             ts = time.time()
-            ssid = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%dT%H:%M:%S")
+            ssid = datetime.datetime.fromtimestamp(ts).strftime(
+                "%Y-%m-%dT%H:%M:%S"
+            )
         self._session_id = ssid
         self._loop_aware_set = set()
 
@@ -934,6 +984,7 @@ class LG:
         self._gather_cache = dict()
 
         lgver = get_lg_ver_type(lg)
+        logger.info("Found LG version: %s", lgver)
 
         if LG_VER_EAGLE == lgver:
             lg = convert_mkn(lg)
@@ -955,8 +1006,6 @@ class LG:
         self._start_list = []
         all_list = []
         stream_output_ports = dict()  # key - port_id, value - construct key
-        input_ports = {}
-        output_ports = {}
         for jd in lg["nodeDataArray"]:
             if (
                 jd["category"] == Categories.COMMENT
@@ -992,18 +1041,13 @@ class LG:
             self.validate_link(src, tgt)
             src.add_output(tgt)
             tgt.add_input(src)
-            # duplicate link from referenced APP into Gathers
-            # if LG_APPREF == lgver and lgk[tgt]['category'] == 'Gather' and \
-            #     'inputApplicationRef' in lgk[tgt].keys():
-            #     if lgk[lk]['inputApplicationRef']:
-            #         ak = lgk[lk]['inputApplicationRef']
-            #         portId = lgk[ak]['inputPorts']
-
             # check stream links
             from_port = lk.get("fromPort", "__None__")
             if stream_output_ports.get(from_port, None) == lk["from"]:
                 lk["is_stream"] = True
-                logger.debug("Found stream from %s to %s", lk["from"], lk["to"])
+                logger.debug(
+                    "Found stream from %s to %s", lk["from"], lk["to"]
+                )
             else:
                 lk["is_stream"] = False
             if "1" == lk.get("loop_aware", "0"):
@@ -1017,7 +1061,9 @@ class LG:
     def validate_link(self, src, tgt):
         # print("validate_link()", src.id, src.is_scatter(), tgt.id, tgt.is_scatter())
         if src.is_scatter() or tgt.is_scatter():
-            prompt = "Remember to specify Input App Type for the Scatter construct!"
+            prompt = (
+                "Remember to specify Input App Type for the Scatter construct!"
+            )
             raise GInvalidLink(
                 "Scatter construct {0} or {1} cannot be linked. {2}".format(
                     src.text, tgt.text, prompt
@@ -1026,7 +1072,9 @@ class LG:
 
         if src.is_loop() or tgt.is_loop():
             raise GInvalidLink(
-                "Loop construct {0} or {1} cannot be linked".format(src.text, tgt.text)
+                "Loop construct {0} or {1} cannot be linked".format(
+                    src.text, tgt.text
+                )
             )
 
         if src.is_gather():
@@ -1044,7 +1092,9 @@ class LG:
         if tgt.is_groupby():
             if src.is_group():
                 raise GInvalidLink(
-                    "GroupBy {0} input must not be a group {1}".format(tgt.id, src.id)
+                    "GroupBy {0} input must not be a group {1}".format(
+                        tgt.id, src.id
+                    )
                 )
             elif len(tgt.inputs) > 0:
                 raise GInvalidLink(
@@ -1059,7 +1109,10 @@ class LG:
                     )
                 )
         elif tgt.is_gather():
-            if not src.jd["category"] in STORAGE_TYPES and not src.is_groupby():
+            if (
+                not src.jd["category"] in STORAGE_TYPES
+                and not src.is_groupby()
+            ):
                 raise GInvalidLink(
                     "Gather {0}'s input {1} should be either a GroupBy or Data".format(
                         tgt.id, src.id
@@ -1143,12 +1196,15 @@ class LG:
                         )
                     for ge in grp_ends:
                         for gs in grp_starts:  # make an artificial circle
-                            ge.add_output(gs)
-                            gs.add_input(ge)
                             lk = dict()
+                            if gs not in ge._outs:
+                                ge.add_output(gs)
+                            if ge not in gs._inputs:
+                                gs.add_input(ge)
                             lk["from"] = ge.id
                             lk["to"] = gs.id
                             self._lg_links.append(lk)
+                            logger.debug("Loop constructed: %s", gs._inputs)
                 else:
                     for (
                         gs
@@ -1198,7 +1254,9 @@ class LG:
                     src_drop = lgn.make_single_drop(miid)
                     self._drop_dict[lgn.id].append(src_drop)
                     if lgn.is_groupby():
-                        self._drop_dict["new_added"].append(src_drop["grp-data_drop"])
+                        self._drop_dict["new_added"].append(
+                            src_drop["grp-data_drop"]
+                        )
                     elif lgn.is_gather():
                         pass
                         # self._drop_dict['new_added'].append(src_drop['gather-data_drop'])
@@ -1207,7 +1265,9 @@ class LG:
         elif lgn.is_mpi():
             for i in range(lgn.dop):
                 miid = "{0}/{1}".format(iid, i)
-                src_drop = lgn.make_single_drop(miid, loop_cxt=lpcxt, proc_index=i)
+                src_drop = lgn.make_single_drop(
+                    miid, loop_cxt=lpcxt, proc_index=i
+                )
                 self._drop_dict[lgn.id].append(src_drop)
         elif lgn.is_service():
             # no action required, inputapp node aleady created and marked with "isService"
@@ -1226,7 +1286,9 @@ class LG:
         for i in range(0, len(l), n):
             yield l[i : i + n]
 
-    def _unroll_gather_as_output(self, slgn, tlgn, sdrops, tdrops, chunk_size, llink):
+    def _unroll_gather_as_output(
+        self, slgn, tlgn, sdrops, tdrops, chunk_size, llink
+    ):
         if slgn.h_level < tlgn.h_level:
             raise GraphException(
                 "Gather {0} has higher h-level than its input {1}".format(
@@ -1300,11 +1362,14 @@ class LG:
             dropSpec_null = dropdict(
                 {
                     "oid": "{0}-{1}-stream".format(
-                        sdrop["oid"], tdrop["oid"].replace(self._session_id, "")
+                        sdrop["oid"],
+                        tdrop["oid"].replace(self._session_id, ""),
                     ),
-                    "type": DropType.PLAIN,
+                    "type": DropType.DATA,
+                    "categoryType": CategoryType.DATA,
                     "storage": Categories.NULL,
                     "nm": "StreamNull",
+                    "text": "StreamNull",
                     "dw": 0,
                 }
             )
@@ -1314,9 +1379,10 @@ class LG:
             tdrop.addStreamingInput(dropSpec_null, IdText="stream")
             self._drop_dict["new_added"].append(dropSpec_null)
         elif s_type in APP_DROP_TYPES:
-            # target is data drop; get name of input port
+            # use name from source and ID from target
+            sIdText = slgn._getIdText("outputPorts")
             tIdText = tlgn._getIdText("inputPorts")
-            sdrop.addOutput(tdrop, IdText=tIdText)
+            sdrop.addOutput(tdrop, IdText=sIdText)
             tdrop.addProducer(sdrop, IdText=tIdText)
             if Categories.BASH_SHELL_APP == s_type:
                 bc = src_drop["command"]
@@ -1332,16 +1398,21 @@ class LG:
             else:  # sdrop is a data drop
                 # there should be only one port, get the name
                 sIdText = slgn._getIdText("outputPorts")
+                # could be multiple ports, need to identify
+                portId = llink["toPort"] if "toPort" in llink else None
+                tIdText = tlgn._getIdText("inputPorts", portId=portId)
                 if llink.get("is_stream", False):
                     logger.debug(
-                        "link stream connection %s to %s", sdrop["oid"], tdrop["oid"]
+                        "link stream connection %s to %s",
+                        sdrop["oid"],
+                        tdrop["oid"],
                     )
                     sdrop.addStreamingConsumer(tdrop, IdText=sIdText)
                     tdrop.addStreamingInput(sdrop, IdText=sIdText)
                 else:
                     # print("not a stream from %s to %s" % (llink['from'], llink['to']))
                     sdrop.addConsumer(tdrop, IdText=sIdText)
-                    tdrop.addInput(sdrop, IdText=sIdText)
+                    tdrop.addInput(sdrop, IdText=tIdText)
             if Categories.BASH_SHELL_APP == t_type:
                 bc = tgt_drop["command"]
                 bc.add_input_param(slgn.id, src_drop["oid"])
@@ -1360,7 +1431,8 @@ class LG:
 
         logger.info(
             "Unroll progress - lgn_to_pgn done %d for session %s",
-                len(self._start_list), self._session_id
+            len(self._start_list),
+            self._session_id,
         )
         self_loop_aware_set = self._loop_aware_set
         for lk in self._lg_links:
@@ -1384,17 +1456,19 @@ class LG:
                     for i, ga_drop in enumerate(sdrops):
                         if ga_drop["oid"] not in self._gather_cache:
                             logger.warning(
-                                "Gather %s Drop not yet in cache, sequentialisation may fail!"
-                                % slgn.text
+                                "Gather %s Drop not yet in cache, sequentialisation may fail!",
+                                slgn.text,
                             )
                             continue
                         j = (i + 1) * slgn.gather_width
                         if j >= tlgn.group.dop and j % tlgn.group.dop == 0:
                             continue
-                        while j < (i + 2) * slgn.gather_width and j < tlgn.group.dop * (
-                            i + 1
-                        ):
-                            gather_input_list = self._gather_cache[ga_drop["oid"]][1]
+                        while j < (
+                            i + 2
+                        ) * slgn.gather_width and j < tlgn.group.dop * (i + 1):
+                            gather_input_list = self._gather_cache[
+                                ga_drop["oid"]
+                            ][1]
                             # TODO merge this code into the function
                             # def _link_drops(self, slgn, tlgn, src_drop, tgt_drop, llink)
                             tIdText = tlgn._getIdText("inputPorts")
@@ -1483,11 +1557,15 @@ class LG:
                         and slgn.h_level > tlgn.h_level
                     ):
                         loop_iter = slgn.group.dop
-                        for i, chunk in enumerate(self._split_list(sdrops, chunk_size)):
+                        for i, chunk in enumerate(
+                            self._split_list(sdrops, chunk_size)
+                        ):
                             for j, sdrop in enumerate(chunk):
                                 # only link drops in the last loop iteration
                                 if j % loop_iter == loop_iter - 1:
-                                    self._link_drops(slgn, tlgn, sdrop, tdrops[i], lk)
+                                    self._link_drops(
+                                        slgn, tlgn, sdrop, tdrops[i], lk
+                                    )
                     elif (
                         tlgn.group is not None
                         and tlgn.group.is_loop()
@@ -1495,22 +1573,34 @@ class LG:
                         and slgn.h_level < tlgn.h_level
                     ):
                         loop_iter = tlgn.group.dop
-                        for i, chunk in enumerate(self._split_list(tdrops, chunk_size)):
+                        for i, chunk in enumerate(
+                            self._split_list(tdrops, chunk_size)
+                        ):
                             for j, tdrop in enumerate(chunk):
                                 # only link drops in the first loop iteration
                                 if j % loop_iter == 0:
-                                    self._link_drops(slgn, tlgn, sdrops[i], tdrop, lk)
+                                    self._link_drops(
+                                        slgn, tlgn, sdrops[i], tdrop, lk
+                                    )
 
                     elif slgn.h_level >= tlgn.h_level:
-                        for i, chunk in enumerate(self._split_list(sdrops, chunk_size)):
+                        for i, chunk in enumerate(
+                            self._split_list(sdrops, chunk_size)
+                        ):
                             # distribute slgn evenly to tlgn
                             for sdrop in chunk:
-                                self._link_drops(slgn, tlgn, sdrop, tdrops[i], lk)
+                                self._link_drops(
+                                    slgn, tlgn, sdrop, tdrops[i], lk
+                                )
                     else:
-                        for i, chunk in enumerate(self._split_list(tdrops, chunk_size)):
+                        for i, chunk in enumerate(
+                            self._split_list(tdrops, chunk_size)
+                        ):
                             # distribute tlgn evenly to slgn
                             for tdrop in chunk:
-                                self._link_drops(slgn, tlgn, sdrops[i], tdrop, lk)
+                                self._link_drops(
+                                    slgn, tlgn, sdrops[i], tdrop, lk
+                                )
             else:  # slgn is not group, but tlgn is group
                 if tlgn.is_groupby():
                     grpby_dict = collections.defaultdict(list)
@@ -1521,7 +1611,8 @@ class LG:
                             # the last bit of iid (current h id) is the local GrougBy key, i.e. inner most loop context id
                             gby = src_ctx[-1]
                             if (
-                                slgn.h_level - 2 == tlgn.h_level and tlgn.h_level > 0
+                                slgn.h_level - 2 == tlgn.h_level
+                                and tlgn.h_level > 0
                             ):  # groupby itself is nested inside a scatter
                                 # group key consists of group context id + inner most loop context id
                                 gctx = "/".join(src_ctx[0:-2])
@@ -1531,7 +1622,9 @@ class LG:
                             gbylist = []
                             if slgn.group.is_groupby():  # a chain of group bys
                                 try:
-                                    src_ctx = gdd["iid"].split("$")[1].split("-")
+                                    src_ctx = (
+                                        gdd["iid"].split("$")[1].split("-")
+                                    )
                                 except IndexError:
                                     raise GraphException(
                                         "The group by hiearchy in the multi-key group by '{0}' is not specified for node '{1}'".format(
@@ -1573,13 +1666,17 @@ class LG:
                     tlgn["type"] = DropType.SERVICE_APP
                 else:
                     raise GraphException(
-                        "Unsupported target group {0}".format(tlgn.jd["category"])
+                        "Unsupported target group {0}".format(
+                            tlgn.jd["category"]
+                        )
                     )
 
         for _, v in self._gather_cache.items():
             input_list = v[1]
             try:
-                output_drop = v[2][0]  # "peek" the first element of the output list
+                output_drop = v[2][
+                    0
+                ]  # "peek" the first element of the output list
             except:
                 continue  # the gather hasn't got output drops, just move on
             llink = v[-1]
@@ -1590,7 +1687,8 @@ class LG:
                 if llink.get("is_stream", False):
                     logger.debug(
                         "link stream connection %s to %s",
-                        data_drop["oid"], output_drop["oid"]
+                        data_drop["oid"],
+                        output_drop["oid"],
                     )
                     data_drop.addStreamingConsumer(output_drop, IdText=sIdText)
                     output_drop.addStreamingInput(data_drop, IdText=sIdText)
@@ -1601,7 +1699,8 @@ class LG:
 
         logger.info(
             "Unroll progress - links done %d for session %s",
-                len(self._lg_links), self._session_id
+            len(self._lg_links),
+            self._session_id,
         )
 
         # clean up extra drops
@@ -1625,7 +1724,7 @@ class LG:
 
         logger.info(
             "Unroll progress - extra drops done for session %s",
-                self._session_id
+            self._session_id,
         )
         ret = []
         for drop_list in self._drop_dict.values():
