@@ -28,38 +28,11 @@ which will then be deployed and monitored by the Physical Graph Manager
 if __name__ == "__main__":
     __package__ = "dlg.dropmake"
 
-import collections
-import datetime
 import json
 import logging
 import string
 import time
-from collections import defaultdict
-from itertools import product
-from typing import ValuesView
 
-import networkx as nx
-import numpy as np
-
-from .scheduler import MySarkarScheduler, DAGUtil, MinNumPartsScheduler, PSOScheduler
-from .utils.bash_parameter import BashCommand
-from ..common import dropdict
-from ..common import Categories, DropType
-from .dm_utils import (
-    LG_APPREF,
-    getNodesKeyDict,
-    get_lg_ver_type,
-    convert_construct,
-    convert_fields,
-    convert_mkn,
-    getAppRefInputs,
-    LG_VER_EAGLE,
-    LG_VER_EAGLE_CONVERTED,
-)
-from .utils.bash_parameter import BashCommand
-from ..common import Categories
-from ..common import STORAGE_TYPES, APP_DROP_TYPES
-from ..common import dropdict
 from dlg.dropmake.lg import LG, GraphException
 from dlg.dropmake.pgt import PGT
 from dlg.dropmake.pgtp import MetisPGTP, MySarkarPGTP, MinNumPartsPGTP, PSOPGTP
@@ -111,9 +84,15 @@ def unroll(lg, oid_prefix=None, zerorun=False, app=None):
             if "sleepTime" in dropspec:
                 dropspec["sleepTime"] = 0
     if app:
+        logger.info("Replacing apps with %s", app)
         for dropspec in drop_list:
             if "app" in dropspec:
                 dropspec["app"] = app
+                dropspec["sleepTime"] = (
+                    dropspec["execution_time"]
+                    if "execution_time" in dropspec
+                    else 2
+                )
     drop_list.append(lg.reprodata)
     return drop_list
 
@@ -211,7 +190,11 @@ def partition(
 
     elif algo == ALGO_MY_SARKAR:
         pgt = MySarkarPGTP(
-            pgt, num_partitions, partition_label, max_dop, merge_parts=could_merge
+            pgt,
+            num_partitions,
+            partition_label,
+            max_dop,
+            merge_parts=could_merge,
         )
 
     elif algo == ALGO_MIN_NUM_PARTS:
@@ -267,9 +250,10 @@ def resource_map(pgt, nodes, num_islands=1, co_host_dim=True):
     if type(pgt[0]) is str:
         pgt = pgt[1]  # remove the graph name TODO: we may want to retain that
     for drop_spec in pgt:
-        nidx = int(drop_spec["node"][1:])  # skip '#'
-        drop_spec["node"] = nm_list[nidx]
-        iidx = int(drop_spec["island"][1:])  # skip '#'
-        drop_spec["island"] = dim_list[iidx]
+        if drop_spec != {}:
+            nidx = int(drop_spec["node"][1:])  # skip '#'
+            drop_spec["node"] = nm_list[nidx]
+            iidx = int(drop_spec["island"][1:])  # skip '#'
+            drop_spec["island"] = dim_list[iidx]
 
     return pgt  # now it's a PG
