@@ -23,7 +23,6 @@ import copy
 import logging
 import os
 import platform
-import random
 import time
 from collections import defaultdict
 
@@ -235,8 +234,8 @@ class Partition(object):
         if len(self._dag.nodes()) == 0:
             return (True, False, False)
 
-        unew = u not in self._dag.node
-        vnew = v not in self._dag.node
+        unew = u not in self._dag.nodes
+        vnew = v not in self._dag.nodes
 
         if DEBUG:
             slow_max = DAGUtil.get_max_antichains(self._dag)
@@ -268,7 +267,7 @@ class Partition(object):
                             mydop, mydop_slow, err_msg
                         )
                     )
-        ret = False if mydop > self._ask_max_dop else True
+        ret = False if mydop > self._ask_max_dop.get('num_cpus', 1) else True
         if unew:
             self.remove(u)
         if vnew:
@@ -284,8 +283,8 @@ class Partition(object):
         #     logger.debug("u = ", u, ", v = ", v, ", partition = ", self.partition_id)
         uw = gu["weight"]
         vw = gv["weight"]
-        unew = u not in self._dag.node
-        vnew = v not in self._dag.node
+        unew = u not in self._dag.nodes
+        vnew = v not in self._dag.nodes
         self._dag.add_node(u, weight=uw, num_cpus=gu["num_cpus"])
         self._dag.add_node(v, weight=vw, num_cpus=gv["num_cpus"])
         self._dag.add_edge(u, v)
@@ -1226,7 +1225,7 @@ class DAGUtil(object):
             oid = drop["oid"]
             myk = i + 1
             tt = drop["type"]
-            if DropType.PLAIN == tt:
+            if DropType.DATA == tt:
                 # if (drop['nm'] == 'StreamNull'):
                 #     obk = 'streamingConsumers'
                 # else:
@@ -1247,18 +1246,20 @@ class DAGUtil(object):
                 G.add_node(
                     myk,
                     weight=tw,
-                    text=drop["nm"],
+                    # text=drop["nm"],
+                    text=drop["text"],
                     dt=dtp,
                     drop_spec=drop,
                     num_cpus=num_cpus,
                 )
             else:
-                G.add_node(myk, weight=tw, text=drop["nm"], dt=dtp, num_cpus=num_cpus)
+                G.add_node(
+                    myk, weight=tw, text=drop["text"], dt=dtp, num_cpus=num_cpus)
             for obk in out_bound_keys:
                 if obk in drop:
                     for oup in drop[obk]:
                         key = list(oup.keys())[0] if isinstance(oup, dict) else oup
-                        if DropType.PLAIN == tt:
+                        if DropType.DATA == tt:
                             G.add_weighted_edges_from(
                                 [(myk, key_dict[key], int(drop["dw"]))]
                             )
@@ -1269,7 +1270,7 @@ class DAGUtil(object):
 
         if fake_super_root:
             super_root = dropdict(
-                {"oid": "-92", "type": DropType.PLAIN, "storage": "null"}
+                {"oid": "-92", "type": DropType.DATA, "storage": "null"}
             )
             super_k = len(drop_list) + 1
             G.add_node(
