@@ -36,8 +36,12 @@ class BashCommand(object):
         cmds: a list such that ' '.join(cmds) looks something like:
                  'python /home/dfms/myclean.py -d %i[-21] -f %i[-3] %o[-2] -v'
         """
-        self._input_map = dict()  # key: logical drop id, value: a list of physical oids
+        self._input_map = (
+            dict()
+        )  # key: logical drop id, value: a list of physical oids
         self._output_map = dict()
+        if len(cmds) > 0 and isinstance(cmds[0], dict):
+            cmds = [list(c.keys())[0] for c in cmds]
         cmd = " ".join(cmds)
         self._cmds = cmd.replace(
             ";", " ; "
