@@ -27,14 +27,16 @@ import json
 import optparse
 import tempfile
 import unittest
+import pkg_resources
 
 from dlg.common.reproducibility.constants import ReproducibilityFlags, ALL_RMODES
 from dlg.translator.tool_commands import dlg_fill, dlg_unroll, dlg_partition, dlg_map
 
 
 def _run_full_workflow(
-    rmode: ReproducibilityFlags, workflow: str, workflow_loc="./", scratch_loc="./"
+        rmode: ReproducibilityFlags, workflow: str, workflow_loc="./", scratch_loc="./"
 ):
+    workflow_loc = pkg_resources.resource_filename("test", workflow_loc)
     lgt = workflow_loc + "/" + workflow + ".graph"
     lgr = scratch_loc + "/" + workflow + "_" + str(rmode.value) + "LG.graph"
     pgs = scratch_loc + "/" + workflow + "_" + str(rmode.value) + "PGS.graph"
@@ -50,7 +52,7 @@ def _run_full_workflow(
     dlg_partition(parser, ["-P", pgs, "-o", pgt, "-f", "newline"])
     parser = optparse.OptionParser()
     dlg_map(
-        parser, ["-P", pgt, "-N", "127.0.0.1, 127.0.0.1", "-o", pgr, "-f", "newline"]
+        parser, ["-P", pgt, "-N", "localhost, localhost", "-o", pgr, "-f", "newline"]
     )
 
 
@@ -75,7 +77,7 @@ class IntegrationNothingTest(unittest.TestCase):
         No data should be present at any level of abstraction, reflected by a null merkleroot.
         """
         graph_name = "HelloSPython"
-        graph_loc = "test/reproducibility/reproGraphs/"
+        graph_loc = "reproducibility/reproGraphs/"
         rmode = ReproducibilityFlags.NOTHING
         _run_full_workflow(
             rmode=rmode,
@@ -84,14 +86,23 @@ class IntegrationNothingTest(unittest.TestCase):
             scratch_loc=self.temp_out.name,
         )
 
-        for filename in ['LG', 'PGS', 'PGT', 'PG']:
+        for filename in ['PGS', 'PGT', 'PG']:
             pgr = (
-                self.temp_out.name + "/" + graph_name + "_" + str(rmode.value) + f"{filename}.graph"
+                    self.temp_out.name + "/" + graph_name + "_" + str(
+                rmode.value) + f"{filename}.graph"
             )
             graph = _read_graph(pgr)
-            self.assertNotIn("reprodata", graph)
             for drop in graph:
-                self.assertNotIn("reprodata", drop)
+                if "reprodata" in drop:
+                    self.assertIn("0", drop["reprodata"]["rmode"])
+                else:
+                    self.assertIn("rmode", drop)
+        lgr = (
+                self.temp_out.name + "/" + graph_name + "_" + str(rmode.value) + "LG.graph"
+        )
+        graph = _read_graph(lgr)
+        for drop in graph["nodeDataArray"]:
+            self.assertIn("reprodata", drop)
 
 
 class IntegrationHelloWorldTest(unittest.TestCase):
@@ -111,7 +122,7 @@ class IntegrationHelloWorldTest(unittest.TestCase):
     """
 
     temp_out = tempfile.TemporaryDirectory("out")
-    graph_loc = "test/reproducibility/reproGraphs/"
+    graph_loc = "reproducibility/reproGraphs/"
     graphs = {
         "HelloWorldBash": {},
         "HelloSBash": {},
@@ -145,7 +156,7 @@ class IntegrationHelloWorldTest(unittest.TestCase):
                     + "_"
                     + str(rmode.value)
                     + "PG.graph"
-                )[-1]["signature"]
+                )[-1][rmode.name]["signature"]
 
     def test_integration_rerun(self):
         """
@@ -492,6 +503,7 @@ class IntegrationSplitRmode(unittest.TestCase):
 
     temp_out = tempfile.TemporaryDirectory("out")
 
+    @unittest.skip("Individual rmodes not yet supported again (needs re-working)")
     def test_split_lgt(self):
         """
         Tests a simple 'hello world' graph (HelloWorldBash) where a single component has
@@ -504,7 +516,7 @@ class IntegrationSplitRmode(unittest.TestCase):
         """
         graph_name = "HelloWorldBashSplit"
         control_graph_name = "HelloWorldBash"
-        graph_loc = "test/reproducibility/reproGraphs/"
+        graph_loc = "reproducibility/reproGraphs/"
         rmode = ReproducibilityFlags.RERUN
         _run_full_workflow(
             rmode=rmode,
@@ -513,7 +525,7 @@ class IntegrationSplitRmode(unittest.TestCase):
             scratch_loc=self.temp_out.name,
         )
         pgr = (
-            self.temp_out.name + "/" + graph_name + "_" + str(rmode.value) + "PG.graph"
+                self.temp_out.name + "/" + graph_name + "_" + str(rmode.value) + "PG.graph"
         )
         _run_full_workflow(
             rmode=rmode,
@@ -522,12 +534,12 @@ class IntegrationSplitRmode(unittest.TestCase):
             scratch_loc=self.temp_out.name,
         )
         pgr_2 = (
-            self.temp_out.name
-            + "/"
-            + control_graph_name
-            + "_"
-            + str(rmode.value)
-            + "PG.graph"
+                self.temp_out.name
+                + "/"
+                + control_graph_name
+                + "_"
+                + str(rmode.value)
+                + "PG.graph"
         )
 
         graph = _read_graph(pgr)
